use beteran_common_rust as bcr;
use beteran_protobuf_rust as bpr;
use prost::Message;

///
pub struct Service {
  connection_broker: nats::asynk::Connection,
  queue_broker: String,
}

impl std::fmt::Debug for Service {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("beteran-backend-server-edge::member_bank_deposit::service::Service")
      .finish()
  }
}

impl Service {
  ///
  pub fn new(connection_broker: nats::asynk::Connection, queue_broker: String) -> Service {
    Service {
      connection_broker,
      queue_broker,
    }
  }

  pub async fn subscribe(&self) -> std::result::Result<(), std::boxed::Box<dyn std::error::Error>> {
    futures::try_join!(
      self.create_member_bank_deposit(),
      self.list_member_bank_deposits(),
      self.get_member_bank_deposit(),
      self.update_member_bank_deposit(),
      self.update_member_bank_deposit_for_state(),
      self.delete_member_bank_deposit(),
    )
    .map(|_| ())
  }

  fn get_client_in_header(
    &self,
    message: &nats::asynk::Message,
  ) -> Result<bpr::models::core::network::Client, bcr::error::rpc::Error> {
    match &message.headers {
      Some(headers) => {
        let client = match headers.get(bpr::c2se::core::network::HEADER_CLIENT) {
          Some(c) => {
            let msg = base64::decode(c).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?;
            bpr::models::core::network::Client::decode(msg.as_slice()).map_err(|e| {
              bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
                message: format!("invalid header: {}", e),
              })
            })?
          }
          None => {
            return Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
              message: "invalid client information".to_string(),
            }));
          }
        };

        Ok(client)
      }
      None => Err(bcr::error::rpc::Error::Parse(bcr::error::rpc::Parse {
        message: "invalid header".to_string(),
      })),
    }
  }

  async fn create_member_bank_deposit(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::member_bank_deposit::SUBJECT_CREATE_MEMBER_BANK_DEPOSIT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_deposit::CreateMemberBankDepositRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_create_member_bank_deposit_req =
          bpr::ss::member_bank_deposit::CreateMemberBankDepositRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_deposit::create_member_bank_deposit_request::Request {
                member_id: "".to_string(),
                name: req.name,
                amount: req.amount,
                memo: req.memo,
              },
            ),
          };

        let ss_create_member_bank_deposit_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_deposit::SUBJECT_CREATE_MEMBER_BANK_DEPOSIT,
            ss_create_member_bank_deposit_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_create_member_bank_deposit_res =
          bpr::ss::member_bank_deposit::CreateMemberBankDepositResponse::decode(
            ss_create_member_bank_deposit_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_create_member_bank_deposit_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_create_member_bank_deposit_res.result {
          message
            .respond(
              bpr::c2se::member_bank_deposit::CreateMemberBankDepositResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_deposit::create_member_bank_deposit_response::Result {
                    member_bank_deposit: r.member_bank_deposit,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_deposit::CreateMemberBankDepositResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn list_member_bank_deposits(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::member_bank_deposit::SUBJECT_LIST_MEMBER_BANK_DEPOSITS,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_deposit::ListMemberBankDepositsRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let ss_list_member_bank_deposits_req = bpr::ss::member_bank_deposit::ListMemberBankDepositsRequest {
          client: Some(client),
          request: Some(bpr::ss::member_bank_deposit::list_member_bank_deposits_request::Request {
            pagination: req.pagination,
            sorts: req.sorts,
  search: match req.search {
      Some(s) => Some(bpr::ss::member_bank_deposit::list_member_bank_deposits_request::request::Search {
        member_id: s.member_id,
        name_like: s.name_like,
        amount: s.amount,
        state: s.state,
        memo_like: s.memo_like,
      }),
      None => None,
  }
          })

        };

        let ss_list_member_bank_deposits_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_deposit::SUBJECT_LIST_MEMBER_BANK_DEPOSITS,
            ss_list_member_bank_deposits_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_list_member_bank_deposits_res =
          bpr::ss::member_bank_deposit::ListMemberBankDepositsResponse::decode(ss_list_member_bank_deposits_res_msg.data.as_slice())
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;

        if let Some(e) = ss_list_member_bank_deposits_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_list_member_bank_deposits_res.result {
          message
            .respond(
              bpr::c2se::member_bank_deposit::ListMemberBankDepositsResponse {
                error: None,
                result: Some(bpr::c2se::member_bank_deposit::list_member_bank_deposits_response::Result {
                  member_bank_deposits: r.member_bank_deposits,
                }),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_deposit::ListMemberBankDepositsResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn get_member_bank_deposit(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::member_bank_deposit::SUBJECT_GET_MEMBER_BANK_DEPOSIT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_deposit::GetMemberBankDepositRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_get_member_bank_deposit_req =
          bpr::ss::member_bank_deposit::GetMemberBankDepositRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_deposit::get_member_bank_deposit_request::Request { id: req.id },
            ),
          };

        let ss_get_member_bank_deposit_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_deposit::SUBJECT_GET_MEMBER_BANK_DEPOSIT,
            ss_get_member_bank_deposit_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_get_member_bank_deposit_res =
          bpr::ss::member_bank_deposit::GetMemberBankDepositResponse::decode(
            ss_get_member_bank_deposit_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_get_member_bank_deposit_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_get_member_bank_deposit_res.result {
          message
            .respond(
              bpr::c2se::member_bank_deposit::GetMemberBankDepositResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_deposit::get_member_bank_deposit_response::Result {
                    member_bank_deposit: r.member_bank_deposit,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_deposit::GetMemberBankDepositResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member_bank_deposit(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::member_bank_deposit::SUBJECT_UPDATE_MEMBER_BANK_DEPOSIT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_deposit::UpdateMemberBankDepositRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_update_member_bank_deposit_req =
          bpr::ss::member_bank_deposit::UpdateMemberBankDepositRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_deposit::update_member_bank_deposit_request::Request {
                id: req.id,
                name: req.name,
                amount: req.amount,
                memo: req.memo,
              },
            ),
          };

        let ss_update_member_bank_deposit_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_deposit::SUBJECT_UPDATE_MEMBER_BANK_DEPOSIT,
            ss_update_member_bank_deposit_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_update_member_bank_deposit_res =
          bpr::ss::member_bank_deposit::UpdateMemberBankDepositResponse::decode(
            ss_update_member_bank_deposit_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_update_member_bank_deposit_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_update_member_bank_deposit_res.result {
          message
            .respond(
              bpr::c2se::member_bank_deposit::UpdateMemberBankDepositResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_deposit::update_member_bank_deposit_response::Result {
                    member_bank_deposit: r.member_bank_deposit,
                  },
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_deposit::UpdateMemberBankDepositResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn update_member_bank_deposit_for_state(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::member_bank_deposit::SUBJECT_UPDATE_MEMBER_BANK_DEPOSIT_FOR_STATE,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_deposit::UpdateMemberBankDepositForStateRequest::decode(message.data.as_slice())
          .map_err(|e| {
            bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
              message: format!("invalid request: {}", e),
            })
          })?;

        let ss_update_member_bank_deposit_for_state_req = bpr::ss::member_bank_deposit::UpdateMemberBankDepositForStateRequest {
          client: Some(client),
          request: Some(bpr::ss::member_bank_deposit::update_member_bank_deposit_for_state_request::Request {
            id: req.id,
            state: req.state,

          })
        };

        let ss_update_member_bank_deposit_for_state_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_deposit::SUBJECT_UPDATE_MEMBER_BANK_DEPOSIT_FOR_STATE,
            ss_update_member_bank_deposit_for_state_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_update_member_bank_deposit_for_state_res = bpr::ss::member_bank_deposit::UpdateMemberBankDepositForStateResponse::decode(
          ss_update_member_bank_deposit_for_state_res_msg.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          })
        })?;

        if let Some(e) = ss_update_member_bank_deposit_for_state_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_update_member_bank_deposit_for_state_res.result {
          message
            .respond(
              bpr::c2se::member_bank_deposit::UpdateMemberBankDepositForStateResponse {
                error: None,
                result: Some(bpr::c2se::member_bank_deposit::update_member_bank_deposit_for_state_response::Result {
                  member_bank_deposit: r.member_bank_deposit,
                }),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_deposit::UpdateMemberBankDepositForStateResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }

  async fn delete_member_bank_deposit(&self) -> Result<(), Box<dyn std::error::Error>> {
    let s = self
      .connection_broker
      .queue_subscribe(
        bpr::c2se::backend::member_bank_deposit::SUBJECT_DELETE_MEMBER_BANK_DEPOSIT,
        self.queue_broker.as_str(),
      )
      .await?;

    while let Some(message) = s.next().await {
      if let Err(e) = async {
        let client = self.get_client_in_header(&message)?;

        let req = bpr::c2se::member_bank_deposit::DeleteMemberBankDepositRequest::decode(
          message.data.as_slice(),
        )
        .map_err(|e| {
          bcr::error::rpc::Error::InvalidRequest(bcr::error::rpc::InvalidRequest {
            message: format!("invalid request: {}", e),
          })
        })?;

        let ss_delete_member_bank_deposit_req =
          bpr::ss::member_bank_deposit::DeleteMemberBankDepositRequest {
            client: Some(client),
            request: Some(
              bpr::ss::member_bank_deposit::delete_member_bank_deposit_request::Request {
                id: req.id,
              },
            ),
          };

        let ss_delete_member_bank_deposit_res_msg = self
          .connection_broker
          .request(
            bpr::ss::member_bank_deposit::SUBJECT_DELETE_MEMBER_BANK_DEPOSIT,
            ss_delete_member_bank_deposit_req.encode_to_vec(),
          )
          .await
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        let ss_delete_member_bank_deposit_res =
          bpr::ss::member_bank_deposit::DeleteMemberBankDepositResponse::decode(
            ss_delete_member_bank_deposit_res_msg.data.as_slice(),
          )
          .map_err(|e| {
            bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
              code: bpr::protobuf::rpc::Error::SERVER_00,
              message: format!("server {}", e),
              data: None,
            })
          })?;

        if let Some(e) = ss_delete_member_bank_deposit_res.error {
          return Err(bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
            code: bpr::protobuf::rpc::Error::SERVER_00,
            message: format!("server {}", e),
            data: None,
          }));
        }

        if let Some(r) = ss_delete_member_bank_deposit_res.result {
          message
            .respond(
              bpr::c2se::member_bank_deposit::DeleteMemberBankDepositResponse {
                error: None,
                result: Some(
                  bpr::c2se::member_bank_deposit::delete_member_bank_deposit_response::Result {},
                ),
              }
              .encode_to_vec(),
            )
            .await
            .map_err(|e| {
              bcr::error::rpc::Error::Server(bcr::error::rpc::Server {
                code: bpr::protobuf::rpc::Error::SERVER_00,
                message: format!("server {}", e),
                data: None,
              })
            })?;
        }

        Ok::<(), bcr::error::rpc::Error>(())
      }
      .await
      {
        message
          .respond(
            bpr::c2se::member_bank_deposit::DeleteMemberBankDepositResponse {
              error: Some(bpr::protobuf::rpc::Error::from(e)),
              result: None,
            }
            .encode_to_vec(),
          )
          .await?;
      }
    }

    Ok(())
  }
}
